@@ -450,7 +450,17 @@ _gpgme_gpgsm_new (GpgsmObject *r_gpgsm)
 	      goto leave;
 	    }
 	}
+
       old_lc = setlocale (LC_CTYPE, NULL);
+      if (old_lc)
+        {
+          old_lc = strdup (old_lc);
+          if (!old_lc)
+            {
+              err = GPGME_Out_Of_Core;
+              goto leave;
+            }
+        }
       dft_lc = setlocale (LC_CTYPE, "");
       if (dft_lc)
 	{
@@ -458,19 +468,32 @@ _gpgme_gpgsm_new (GpgsmObject *r_gpgsm)
 	    err = mk_error (Out_Of_Core);
 	  else
 	    {
-	      err = assuan_transact (gpgsm->assuan_ctx, optstr, NULL, NULL, NULL, NULL, NULL,
-				     NULL);
+	      err = assuan_transact (gpgsm->assuan_ctx, optstr, NULL, NULL,
+                                     NULL, NULL, NULL, NULL);
 	      free (optstr);
 	      if (err)
 		err = map_assuan_error (err);
 	    }
 	}
       if (old_lc)
-	setlocale (LC_CTYPE, old_lc);
+        {
+          setlocale (LC_CTYPE, old_lc);
+          free (old_lc);
+        }
       if (err)
 	goto leave;
 
+
       old_lc = setlocale (LC_MESSAGES, NULL);
+      if (old_lc)
+        {
+          old_lc = strdup (old_lc);
+          if (!old_lc)
+            {
+              err = GPGME_Out_Of_Core;
+              goto leave;
+            }
+        }
       dft_lc = setlocale (LC_MESSAGES, "");
       if (dft_lc)
 	{
@@ -486,7 +509,10 @@ _gpgme_gpgsm_new (GpgsmObject *r_gpgsm)
 	    }
 	}
       if (old_lc)
-	setlocale (LC_MESSAGES, old_lc);
+        {
+          setlocale (LC_MESSAGES, old_lc);
+          free (old_lc);
+        }
       if (err)
 	goto leave;
     }
