@@ -88,20 +88,19 @@ append_xml_keylistinfo (GpgmeData *rdh, char *args)
 
 
 static void
-keylist_status_handler (GpgmeCtx ctx, GpgStatusCode code, char *args)
+keylist_status_handler (GpgmeCtx ctx, GpgmeStatusCode code, char *args)
 {
   if (ctx->error)
     return;
   test_and_allocate_result (ctx, keylist);
 
-  switch (code)
+   switch (code)
     {
-    case STATUS_TRUNCATED:
+    case GPGME_STATUS_TRUNCATED:
       ctx->result.keylist->truncated = 1;
       break;
 
-    case STATUS_EOF:
-      finish_key (ctx);
+    case GPGME_STATUS_EOF:
       if (ctx->result.keylist->truncated)
         append_xml_keylistinfo (&ctx->result.keylist->xmlinfo, "1");
       if (ctx->result.keylist->xmlinfo)
@@ -149,18 +148,24 @@ set_mainkey_trust_info (GpgmeKey key, const char *s)
 static void
 set_userid_flags (GpgmeKey key, const char *s)
 {
+  struct user_id_s *u = key->uids;
+
+  assert (u);
+  while (u->next)
+    u = u->next;
+
   /* Look at letters and stop at the first digit.  */
   for (; *s && !my_isdigit (*s); s++)
     {
       switch (*s)
 	{
-	case 'r': key->uids->revoked  = 1; break;
-	case 'i': key->uids->invalid  = 1; break;
+	case 'r': u->revoked  = 1; break;
+	case 'i': u->invalid  = 1; break;
 
-	case 'n': key->uids->validity = GPGME_VALIDITY_NEVER; break;
-	case 'm': key->uids->validity = GPGME_VALIDITY_MARGINAL; break;
-	case 'f': key->uids->validity = GPGME_VALIDITY_FULL; break;
-	case 'u': key->uids->validity = GPGME_VALIDITY_ULTIMATE; break;
+	case 'n': u->validity = GPGME_VALIDITY_NEVER; break;
+	case 'm': u->validity = GPGME_VALIDITY_MARGINAL; break;
+	case 'f': u->validity = GPGME_VALIDITY_FULL; break;
+	case 'u': u->validity = GPGME_VALIDITY_ULTIMATE; break;
         }
     }
 }
@@ -250,6 +255,8 @@ keylist_colon_handler (GpgmeCtx ctx, char *line)
   const char *trust_info = NULL;
   struct subkey_s *sk = NULL;
 
+  DEBUG3 ("keylist_colon_handler ctx=%p, key=%p, line=%s\n", ctx, key,
+          line? line: "(null)");
   if (ctx->error)
     return;
   if (!line)
@@ -374,7 +381,7 @@ keylist_colon_handler (GpgmeCtx ctx, char *line)
                 break;
 	    case 4: /* pubkey algo */
 	      i = atoi (p);
-	      if (i > 1 && i < 128)
+	      if (i >= 1 && i < 128)
 		key->keys.key_algo = i;
 	      break;
               case 5: /* long keyid */
@@ -388,7 +395,7 @@ keylist_colon_handler (GpgmeCtx ctx, char *line)
 	      key->keys.expires_at = parse_timestamp (p);
 	      break;
 	    case 8: /* X.509 serial number */
-              if (rectype == RT_CRT)
+              if (rectype == RT_CRT || rectype == RT_CRS)
                 {
                   key->issuer_serial = xtrystrdup (p);
 		  if (!key->issuer_serial)
@@ -398,14 +405,12 @@ keylist_colon_handler (GpgmeCtx ctx, char *line)
 	    case 9: /* ownertrust */
               set_ownertrust (key, p);
 	      break;
-	    case 10: /* not used for gpg due to --fixed-list-mode option 
-			but gpgsm stores the issuer name */
-              if (rectype == RT_CRT)
-                {
-                  key->issuer_name = xtrystrdup (p);
-		  if (!key->issuer_name)
-		    ctx->error = mk_error (Out_Of_Core);
-                }
+	    case 10:
+	      /* Not used for gpg due to --fixed-list-mode option but
+		 GPGSM stores the issuer name.  */
+              if (rectype == RT_CRT || rectype == RT_CRS)
+		if (_gpgme_decode_c_string (p, &key->issuer_name))
+		  ctx->error = mk_error (Out_Of_Core);
 	      break;
 	    case 11: /* signature class  */
 	      break;
@@ -431,7 +436,7 @@ keylist_colon_handler (GpgmeCtx ctx, char *line)
                 break;
               case 4: /* pubkey algo */
                 i = atoi (p);
-                if (i > 1 && i < 128)
+                if (i >= 1 && i < 128)
 		  sk->key_algo = i;
                 break;
               case 5: /* long keyid */
@@ -442,6 +447,7 @@ keylist_colon_handler (GpgmeCtx ctx, char *line)
                 sk->timestamp = parse_timestamp (p);
                 break;
               case 7: /* expiration time (seconds) */
+                sk->expires_at = parse_timestamp (p);
                 break;
               case 8: /* reserved (LID) */
                 break;
@@ -473,7 +479,7 @@ keylist_colon_handler (GpgmeCtx ctx, char *line)
 	      else
 		{
 		  if (trust_info)
-		  set_userid_flags (key, trust_info);
+                    set_userid_flags (key, trust_info);
 		}
 	      pend = NULL;  /* we can stop here */
 	      break;
@@ -514,35 +520,45 @@ static void
 finish_key (GpgmeCtx ctx)
 {
   GpgmeKey key = ctx->tmp_key;
-  struct key_queue_item_s *q, *q2;
+
+  ctx->tmp_key = NULL;
 
   if (key)
+    _gpgme_engine_io_event (ctx->engine, GPGME_EVENT_NEXT_KEY, key);
+}
+
+
+void
+_gpgme_op_keylist_event_cb (void *data, GpgmeEventIO type, void *type_data)
+{
+  GpgmeCtx ctx = (GpgmeCtx) data;
+  GpgmeKey key = (GpgmeKey) type_data;
+  struct key_queue_item_s *q, *q2;
+
+  assert (type == GPGME_EVENT_NEXT_KEY);
+
+  _gpgme_key_cache_add (key);
+
+  q = xtrymalloc (sizeof *q);
+  if (!q)
     {
-      ctx->tmp_key = NULL;
-        
-      _gpgme_key_cache_add (key);
-        
-      q = xtrymalloc (sizeof *q);
-      if (!q)
-	{
-	  gpgme_key_release (key);
-	  ctx->error = mk_error (Out_Of_Core);
-	  return;
-        }
-      q->key = key;
-      q->next = NULL;
-      /* FIXME: Lock queue.  Use a tail pointer?  */
-      if (!(q2 = ctx->key_queue))
-	ctx->key_queue = q;
-      else
-	{
-	  for (; q2->next; q2 = q2->next)
-	    ;
-	  q2->next = q;
-        }
-      ctx->key_cond = 1;
-      /* FIXME: Unlock queue.  */
+      gpgme_key_release (key);
+      ctx->error = mk_error (Out_Of_Core);
+      return;
     }
+  q->key = key;
+  q->next = NULL;
+  /* FIXME: Lock queue.  Use a tail pointer?  */
+  if (!(q2 = ctx->key_queue))
+    ctx->key_queue = q;
+  else
+    {
+      for (; q2->next; q2 = q2->next)
+	;
+      q2->next = q;
+    }
+  ctx->key_cond = 1;
+  /* FIXME: Unlock queue.  */
 }
 
 
@@ -563,7 +579,7 @@ gpgme_op_keylist_start (GpgmeCtx ctx, const char *pattern, int secret_only)
 {
   GpgmeError err = 0;
 
-  err = _gpgme_op_reset (ctx, 0);
+  err = _gpgme_op_reset (ctx, 2);
   if (err)
     goto leave;
 
@@ -618,11 +634,12 @@ gpgme_op_keylist_ext_start (GpgmeCtx ctx, const char *pattern[],
 {
   GpgmeError err = 0;
 
-  err = _gpgme_op_reset (ctx, 0);
+  err = _gpgme_op_reset (ctx, 2);
   if (err)
     goto leave;
 
   gpgme_key_release (ctx->tmp_key);
+  ctx->tmp_key = NULL;
 
   _gpgme_engine_set_status_handler (ctx->engine, keylist_status_handler, ctx);
   err = _gpgme_engine_set_colon_line_handler (ctx->engine,
@@ -682,9 +699,21 @@ gpgme_op_keylist_next (GpgmeCtx ctx, GpgmeKey *r_key)
 
   if (!ctx->key_queue)
     {
-      _gpgme_wait_on_condition (ctx, 1, &ctx->key_cond);
-      if (ctx->error)
-	return ctx->error;
+      GpgmeError err = _gpgme_wait_on_condition (ctx, &ctx->key_cond);
+      if (err)
+	{
+	  ctx->pending = 0;
+	  return err;
+	}
+      if (!ctx->pending)
+	{
+	  /* The operation finished.  Because not all keys might have
+	     been returned to the caller yet, we just reset the
+	     pending flag to 1.  This will cause us to call
+	     _gpgme_wait_on_condition without any active file
+	     descriptors, but that is a no-op, so it is safe.  */
+	  ctx->pending = 1;
+	}
       if (!ctx->key_cond)
 	{
 	  ctx->pending = 0;
